CREATE OR REPLACE PROCEDURE BALANCETE_AUX_FIL(
   P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFILIAL IN VARCHAR2,
   P_CDCONTINI IN VARCHAR2,
   P_CDCONTFIN IN VARCHAR2,
   P_CDCLIENTE IN VARCHAR2,
   P_CDFORNECED IN VARCHAR2,
   P_CDREDUCONT IN VARCHAR2,
   P_DTINI IN LANCCONT.DTLANCCONT%TYPE,
   P_DTFIN IN LANCCONT.DTLANCCONT%TYPE,
   P_NRBALANCETE OUT NUMBER
)
AS
   V_NOVOCODIGO VARCHAR2(20);
   V_DATAINI DATE;
   V_NRBALANCETE NUMBER;
   V_NRSEQ NUMBER;
   V_COUNT NUMBER;
   FUNCTION SEQUENCIAL RETURN NUMBER
   IS
      V_TEMP VARCHAR2(20);
   BEGIN
      NOVO_CODIGO_NT('SEQBALANCOMFIL', V_TEMP);
      RETURN(TO_NUMBER(V_TEMP));
   END;
BEGIN
   NOVO_CODIGO_NT('BALANCOMFIL', V_NOVOCODIGO);
   V_NRBALANCETE := TO_NUMBER(V_NOVOCODIGO);
   V_NRSEQ := SEQUENCIAL;
   BEGIN
      SELECT MIN(DTLANCCONT)
        INTO V_DATAINI
        FROM LANCCONT
       WHERE CDEMPRESA = P_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DATAINI := TO_DATE('01/01/2000');
   END;

   /* CURSOIR RESPONSVEL POR GERAR OS SALDOS ANTERIORES.
      UNIONS FORAM USADOS PARA SEPARAR OS SALDOS POR TIPO COMERCIAL: CLIENTE, FORNECEDOR E BANCO */
   FOR C_SALDOS IN
      (SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'C' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'C'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDCLIENTE, L.CDCONTCOME) > 0) OR ('T' = P_CDCLIENTE))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'F' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'F'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDFORNECED, L.CDCONTCOME) > 0) OR ('T' = P_CDFORNECED))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'B' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'B'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDREDUCONT, L.CDCONTCOME) > 0) OR ('T' = P_CDREDUCONT))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0) LOOP
        
      V_NRSEQ := SEQUENCIAL;
      INSERT INTO BALANCOMFIL
         (NRBALCOMFIL, NRSEQBALCOM, CDEMPRESA, CDFILIAL, CDCONTCTBL, CDCONTCOME,
          IDTIPOCOME, VRSALDANT, VRDEBITO, VRCREDITO, VRSALDATUAL)
      VALUES
         (V_NRBALANCETE, V_NRSEQ, C_SALDOS.CDEMPRESA, C_SALDOS.CDFILIAL, C_SALDOS.CDCONTCTBL, C_SALDOS.CDCONTCOME,
         C_SALDOS.IDTIPO, C_SALDOS.VRSALDANT, 0, 0, 0);
   END LOOP;

   /* CURSOR RESPONSVEL POR REALIZAR O SOMATRIO DAS PARTIDAS DE DBITO */
   FOR C_DEB IN
      (SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRDEBITO, 'B' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'B'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDREDUCONT, L.CDCONTCOME) > 0) OR ('T' = P_CDREDUCONT))
          AND LC.VRLANCCUST < 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME  

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRDEBITO, 'C' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'C'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDCLIENTE, L.CDCONTCOME) > 0) OR ('T' = P_CDCLIENTE))
          AND LC.VRLANCCUST < 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRDEBITO, 'F' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'F'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDFORNECED, L.CDCONTCOME) > 0) OR ('T' = P_CDFORNECED))
          AND LC.VRLANCCUST < 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME) LOOP
      V_COUNT := 0;
      BEGIN
         SELECT COUNT(*)
           INTO V_COUNT
           FROM BALANCOMFIL
          WHERE NRBALCOMFIL = V_NRBALANCETE
            AND CDCONTCTBL = C_DEB.CDCONTCTBL
            AND CDCONTCOME = C_DEB.CDCONTCOME
            AND CDFILIAL = C_DEB.CDFILIAL
            AND IDTIPOCOME = C_DEB.IDTIPO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNT := 0;
      END;

      IF V_COUNT = 0 THEN
         V_NRSEQ := SEQUENCIAL;
         INSERT INTO BALANCOMFIL
            (NRBALCOMFIL, NRSEQBALCOM, CDEMPRESA, CDFILIAL, CDCONTCTBL, CDCONTCOME,
             IDTIPOCOME, VRSALDANT, VRDEBITO, VRCREDITO, VRSALDATUAL)
         VALUES
            (V_NRBALANCETE, V_NRSEQ, C_DEB.CDEMPRESA, C_DEB.CDFILIAL, C_DEB.CDCONTCTBL, C_DEB.CDCONTCOME,
            C_DEB.IDTIPO, 0, 0, 0, 0);
      END IF;
      UPDATE BALANCOMFIL
         SET VRDEBITO = C_DEB.VRDEBITO
       WHERE NRBALCOMFIL = V_NRBALANCETE
         AND CDCONTCTBL = C_DEB.CDCONTCTBL
         AND CDCONTCOME = C_DEB.CDCONTCOME
         AND IDTIPOCOME = C_DEB.IDTIPO;
   END LOOP;

   /* CURSOR RESPONSVEL POR REALIZAR O SOMATRIO DAS PARTIDAS DE CRDITO */
   FOR C_CRED IN
      (SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRCREDITO, 'B' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'B'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDREDUCONT, L.CDCONTCOME) > 0) OR ('T' = P_CDREDUCONT))
          AND LC.VRLANCCUST > 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRCREDITO, 'C' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'C'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDCLIENTE, L.CDCONTCOME) > 0) OR ('T' = P_CDCLIENTE))
          AND LC.VRLANCCUST > 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRCREDITO, 'F' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'F'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDFORNECED, L.CDCONTCOME) > 0) OR ('T' = P_CDFORNECED))
          AND LC.VRLANCCUST > 0
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME) LOOP

      V_COUNT := 0;
      BEGIN
         SELECT COUNT(*)
           INTO V_COUNT
           FROM BALANCOMFIL
          WHERE NRBALCOMFIL = V_NRBALANCETE
            AND CDCONTCTBL = C_CRED.CDCONTCTBL
            AND CDCONTCOME = C_CRED.CDCONTCOME
            AND CDFILIAL = C_CRED.CDFILIAL
            AND IDTIPOCOME = C_CRED.IDTIPO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNT := 0;
      END;

      IF V_COUNT = 0 THEN
         V_NRSEQ := SEQUENCIAL;
         INSERT INTO BALANCOMFIL
            (NRBALCOMFIL, NRSEQBALCOM, CDEMPRESA, CDFILIAL, CDCONTCTBL, CDCONTCOME,
             IDTIPOCOME, VRSALDANT, VRDEBITO, VRCREDITO, VRSALDATUAL)
         VALUES
            (V_NRBALANCETE, V_NRSEQ, C_CRED.CDEMPRESA, C_CRED.CDFILIAL, C_CRED.CDCONTCTBL, C_CRED.CDCONTCOME,
            C_CRED.IDTIPO, 0, 0, 0, 0);
      END IF;
      UPDATE BALANCOMFIL
         SET VRCREDITO = C_CRED.VRCREDITO
       WHERE NRBALCOMFIL = V_NRBALANCETE
         AND CDCONTCTBL = C_CRED.CDCONTCTBL
         AND CDCONTCOME = C_CRED.CDCONTCOME
         AND IDTIPOCOME = C_CRED.IDTIPO;
   END LOOP;

   /* ATUALIZAO DO SALDO ATUAL */
   UPDATE BALANCOMFIL
      SET VRSALDATUAL = VRSALDANT + VRDEBITO + VRCREDITO
    WHERE NRBALCOMFIL = V_NRBALANCETE;
   P_NRBALANCETE := V_NRBALANCETE;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('BALANCETE_AUX_FIL','5.85.000'); END;
/